@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2014 Red Hat.
+ * Copyright (c) 2013-2015 Red Hat.
  * 
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -281,8 +281,12 @@ __pmServerCreatePIDFile(const char *spec, int verbose)
     }
     atexit(pidonexit);
     fprintf(pidfile, "%" FMT_PID, getpid());
+#ifdef HAVE_FCHMOD
+    (void)fchmod(fileno(pidfile), S_IRUSR | S_IRGRP | S_IROTH);
+#else
+    (void)chmod(pidpath, S_IRUSR | S_IRGRP | S_IROTH);
+#endif
     fclose(pidfile);
-    chmod(pidpath, S_IRUSR | S_IRGRP | S_IROTH);
     return 0;
 }
 
@@ -618,8 +622,9 @@ OpenRequestPorts(__pmFdSet *fdset, int backlog)
 	}
 	if (portsOpened > 0) {
 	    /* Advertise our presence on the network, if requested. */
-	    if (serviceSpec != NULL) {
-		rp->presence =  __pmServerAdvertisePresence(serviceSpec,
+	    if (serviceSpec != NULL &&
+		__pmServerHasFeature(PM_SERVER_FEATURE_DISCOVERY)) {
+		rp->presence = __pmServerAdvertisePresence(serviceSpec,
 							    rp->port);
 	    }
 	}
@@ -884,7 +889,7 @@ __pmSecureServerHandshake(int fd, int flags, __pmHashCtl *attrs)
     /*
      * CREDS_REQD is a special case that does not need a secure server
      * provided we've connected on a unix domain socket
-     * */
+     */
     if ((flags & PDU_FLAG_CREDS_REQD) != 0 && __pmHashSearch(PCP_ATTR_USERID, attrs) != NULL) {
 #ifdef PCP_DEBUG
 	if (pmDebug & DBG_TRACE_AUTH)
@@ -892,7 +897,13 @@ __pmSecureServerHandshake(int fd, int flags, __pmHashCtl *attrs)
 #endif
 	return 0;
     }
-    /* otherwise the flags are not expected */
+    /* remove all of the known good flags */
+    flags &= ~(PDU_FLAG_SECURE | PDU_FLAG_SECURE_ACK | PDU_FLAG_COMPRESS |
+	       PDU_FLAG_AUTH | PDU_FLAG_CREDS_REQD | PDU_FLAG_CONTAINER);
+    if (!flags)
+	return 0;
+
+    /* any remaining flags are unexpected */
 #ifdef PCP_DEBUG
     if (pmDebug & DBG_TRACE_AUTH)
 	fprintf(stderr, "bad\n");
@@ -941,11 +952,19 @@ int
 __pmServerSetFeature(__pmServerFeature wanted)
 {
     if (wanted == PM_SERVER_FEATURE_DISCOVERY ||
-        wanted == PM_SERVER_FEATURE_CREDS_REQD ||
+	wanted == PM_SERVER_FEATURE_CREDS_REQD ||
 	wanted == PM_SERVER_FEATURE_UNIX_DOMAIN) {
 	server_features |= (1 << wanted);
 	return 1;
     }
+    if (wanted == PM_SERVER_FEATURE_CONTAINERS) {
+#if defined(HAVE_SETNS)
+	server_features |= (1 << wanted);
+	return 1;
+#else
+	return 0;
+#endif
+    }
     return __pmSecureServerSetFeature(wanted);
 }
 
@@ -959,6 +978,7 @@ __pmServerHasFeature(__pmServerFeature query)
 	sts = (strcmp(__pmGetAPIConfig("ipv6"), "true") == 0);
 	break;
     case PM_SERVER_FEATURE_DISCOVERY:
+    case PM_SERVER_FEATURE_CONTAINERS:
     case PM_SERVER_FEATURE_CREDS_REQD:
     case PM_SERVER_FEATURE_UNIX_DOMAIN:
 	if (server_features & (1 << query))
