@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2014 Red Hat.
+ * Copyright (c) 2013-2015 Red Hat.
  * Copyright (c) 2010 Aconex.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -79,6 +79,9 @@ typedef struct {
     __uint64_t		recent_rotated_file;
     __uint64_t		recent_scanned_anon;
     __uint64_t		recent_scanned_file;
+    __uint64_t		usage;
+    __uint64_t		limit;
+    __uint64_t		failcnt;
 } cgroup_memory_t;
 
 enum {
@@ -118,6 +121,10 @@ enum {
     CG_MEMORY_STAT_RECENT_ROTATED_FILE	= 61,
     CG_MEMORY_STAT_RECENT_SCANNED_ANON	= 62,
     CG_MEMORY_STAT_RECENT_SCANNED_FILE	= 63,
+
+    CG_MEMORY_USAGE_IN_BYTES		= 80,
+    CG_MEMORY_LIMIT_IN_BYTES		= 81,
+    CG_MEMORY_FAILCNT			= 82,
 };
 
 typedef struct {
@@ -261,9 +268,10 @@ enum {
  */
 typedef void (*cgroup_setup_t)(void);
 typedef void (*cgroup_refresh_t)(const char *, const char *);
-extern void refresh_cgroups(const char *,
+extern void refresh_cgroups(const char *, const char *, int,
 			    cgroup_setup_t, cgroup_refresh_t);
 extern char *cgroup_find_subsys(pmInDom, filesys_t *);
+extern int cgroup_mounts_subsys(const char *, char *, int);
 
 extern void refresh_cgroup_subsys(void);
 extern void refresh_cgroup_filesys(void);
