@@ -1,6 +1,6 @@
 #! /bin/sh
 #
-# Copyright (c) 2013-2014 Red Hat.
+# Copyright (c) 2013-2015 Red Hat.
 # Copyright (c) 1995-2000,2003 Silicon Graphics, Inc.  All Rights Reserved.
 # 
 # This program is free software; you can redistribute it and/or modify it
@@ -21,7 +21,7 @@
 . $PCP_DIR/etc/pcp.env
 . $PCP_SHARE_DIR/lib/rc-proc.sh
 
-PMLOGGER=pmlogger
+PMLOGGER="$PCP_BINADM_DIR/pmlogger"
 PMLOGCONF="$PCP_BINADM_DIR/pmlogconf"
 
 # error messages should go to stderr, not the GUI notifiers
@@ -193,19 +193,6 @@ _unlock()
     echo >$tmp/lock
 }
 
-_get_ino()
-{
-    # get inode number for $1
-    # throw away stderr (and return '') in case $1 has been removed by now
-    #
-    stat "$1" 2>/dev/null \
-    | sed -n '/Device:[ 	].*[ 	]Inode:/{
-s/Device:[ 	].*[ 	]Inode:[ 	]*//
-s/[ 	].*//
-p
-}'
-}
-
 _get_configfile()
 {
     # extract the pmlogger configuration file (-c) from a list of arguments
@@ -235,7 +222,7 @@ _configure_pmlogger()
         then
             # pmlogconf file that we own, see if re-generation is needed
             cp "$configfile" $tmp/pmlogger
-            if $PMLOGCONF -c -q -h $hostname $tmp/pmlogger >$tmp/diag 2>&1
+            if $PMLOGCONF -r -c -q -h $hostname $tmp/pmlogger >$tmp/diag 2>&1
             then
 		if grep 'No changes' $tmp/diag >/dev/null 2>&1
                 then
@@ -301,6 +288,23 @@ _get_logfile()
     done
 }
 
+_get_primary_logger_pid()
+{
+    file="$PCP_TMP_DIR/pmlogger/primary"
+    if [ ! -L "$file" ]
+    then
+	pid=''
+    elif which realpath >/dev/null 2>&1
+    then
+	pri=`readlink $file`
+	pid=`basename "$pri"`
+    else
+	pri=`ls -l "$file" | sed -e 's/.*-> //'`
+	pid=`basename "$pri"`
+    fi
+    echo "$pid"
+}
+
 _check_archive()
 {
     if [ ! -e "$logfile" ]
@@ -608,33 +612,15 @@ s/^\([A-Za-z][A-Za-z0-9_]*\)=/export \1; \1=/p
         # in the primary logger case), but it *does* matter for pmlogconf.
         host=local:
 
-	if test -f "$PCP_TMP_DIR/pmlogger/primary"
+	if test -e "$PCP_TMP_DIR/pmlogger/primary"
 	then
 	    if $VERY_VERBOSE
 	    then 
 		_host=`sed -n 2p <"$PCP_TMP_DIR/pmlogger/primary"`
 		_arch=`sed -n 3p <"$PCP_TMP_DIR/pmlogger/primary"`
-		$PCP_ECHO_PROG $PCP_ECHO_N "... try $PCP_TMP_DIR/pmlogger/primary: host=$_host arch=$_arch""$PCP_ECHO_C"
+		echo "... try $PCP_TMP_DIR/pmlogger/primary: host=$_host arch=$_arch"
 	    fi
-	    primary_inode=`_get_ino $PCP_TMP_DIR/pmlogger/primary`
-	    $VERY_VERBOSE && echo primary_inode=$primary_inode
-	    for file in $PCP_TMP_DIR/pmlogger/*
-	    do
-		case "$file"
-		in
-		    */primary|*\*)
-			;;
-		    */[0-9]*)
-			inode=`_get_ino "$file"`
-			$VERY_VERBOSE && echo $file inode=$inode
-			if [ "$primary_inode" = "$inode" ]
-			then
-			    pid="`echo $file | sed -e 's/.*\/\([^/]*\)$/\1/'`"
-			    break
-			fi
-			;;
-		esac
-	    done
+	    pid=`_get_primary_logger_pid`
 	    if [ -z "$pid" ]
 	    then
 		if $VERY_VERBOSE
